using System.Reflection;
using System.Runtime.InteropServices;
using Rhino.PlugIns;

// General plugin information
[assembly: AssemblyTitle("RhinoMcpPlugin")]
[assembly: AssemblyDescription("Rhino plugin that hosts an MCP server for AI-assisted 3D modeling")]
[assembly: AssemblyConfiguration("")]
[assembly: AssemblyCompany("")]
[assembly: AssemblyProduct("RhinoMcpPlugin")]
[assembly: AssemblyCopyright("Copyright © 2024")]
[assembly: AssemblyTrademark("")]
[assembly: AssemblyCulture("")]

// Plugin Guid - unique identifier
[assembly: Guid("A3C5A369-C051-4732-B1A7-F3C1C8A9EC2D")]

// Plugin version
[assembly: AssemblyVersion("1.0.0.0")]
[assembly: AssemblyFileVersion("1.0.0.0")]

// The plugin type
[assembly: PlugInDescription(DescriptionType.Address, "")]
[assembly: PlugInDescription(DescriptionType.Country, "")]
[assembly: PlugInDescription(DescriptionType.Email, "")]
[assembly: PlugInDescription(DescriptionType.Organization, "")]
[assembly: PlugInDescription(DescriptionType.Phone, "")]
[assembly: PlugInDescription(DescriptionType.WebSite, "")] 